import winappdbg
from winappdbg import Color
from winappdbg import EventHandler
from winappdbg.win32 import *
import sys
import time
import urlparse

debug_target = 'jusched.exe'

def print_script(str):
	Color.white()
	Color.light()
	print '[%s] %s' % (time.strftime("%Y-%m-%d %H:%M:%S",time.gmtime()), str)
	Color.reset()
	return

def print_modification(str):
	Color.red()
	Color.light()
	print '[%s] %s' % (time.strftime("%Y-%m-%d %H:%M:%S",time.gmtime()), str)
	Color.reset()
	return

def print_process(str):
	Color.cyan()
	Color.light()
	print '[%s] %s' % (time.strftime("%Y-%m-%d %H:%M:%S",time.gmtime()), str)
	Color.reset()	
	
def get_state(event):
	process = event.get_process()
	thread = event.get_thread()
	context = thread.get_context()
	return process, thread, context

def setup_call(event):
	"""
	Breakpoint: 0x00408F93
	Sets up the call to search_process_memory (0x00407CC0)
	Returns to the end of WinMain, at 0040919A to somewhat
	cleanly exit.
	"""
	process, thread, context = get_state(event)
	
	esp = context['Esp']
	
	print_modification('Allocating/clearing space on stack for two 3-DWORD data structure arguments')
	esp = esp - 24
	p_structure_1 = esp
	p_structure_2 = esp + 12
	thread.set_register('Esp', esp)
	process.write(esp, '\x00'*24)
	
	print_modification('Setting third argument (Stack arg) to point to second data structure at %s' % hex(p_structure_2))
	esp = esp - 4
	thread.set_register('Esp', esp)
	process.write_dword(esp, p_structure_2)

	print_modification('Setting second argument (EDX) to point to first data structure at %s' % hex(p_structure_1))
	thread.set_register('Edx', p_structure_1)
	
	print_modification('Setting first argument (ECX) to PID %s' % sys.argv[1])
	thread.set_register('Ecx', int(sys.argv[1]))
	
	print_modification('Setting return address to cleanly exit')
	esp = esp - 4
	thread.set_register('Esp', esp)
	process.write_dword(esp, 0x0409197)
	
	print_modification('Setting EIP to beginning of process search function')
	thread.set_register('Eip', 0x00407CC0)
	
	print_process('Process search starting')
	
	return
	
def function_return(event):
	"""
	Breakpoint: 0x00407E29
	...for when the process search function is done
	"""
	process, thread, context = get_state(event)
	print_process('Process search done')
	return
	
def open_process(event):
	"""
	Breakpoint: 0x00407D06
	...for when the process search function is done
	"""
	process, thread, context = get_state(event)
	ecx = context['Ecx']
	target_process = winappdbg.Process(ecx)
	print_process('Opening PID %i (%s)' % (ecx, target_process.get_command_line()))
	return
	
def reading_region(event):
	"""
	Breakpoint: 0x00407D4F
	Region determined to be one that is suitable to read and scan for track data
	"""
	process, thread, context = get_state(event)
	esi = context['Esi']
	ebp = context['Ebp']
	size = process.peek_dword(ebp+0x34)
	print_process('  Reading: 0x%08x to 0x%08x' % (esi, esi + size))
	return

def test(event):
	process, thread, context = get_state(event)
	ebp = context['Ebp']
	ds1 = process.peek_dword(ebp-0x48)
	ds2 = process.peek_dword(ebp-0x44)
	print 'Data structure 1:' 
	print hex(process.peek_dword(ds1))
	print hex(process.peek_dword(ds1+4))
	print hex(process.peek_dword(ds1+8))
	print 'Data structure 2:' 
	print hex(process.peek_dword(ds2))
	print hex(process.peek_dword(ds2+4))
	print hex(process.peek_dword(ds2+8))
	return

	
breakpoints = [(0x00408F93, setup_call),
			   (0x00407E29, function_return),
			   (0x00407D06, open_process),
			   (0x00407D4F, reading_region),
			   (0x00407DCA, test),
			   ]

#######

class JackPOSEventHandler(EventHandler):
	apiHooks = {
		'kernel32.dll' : [
			('Sleep',(DWORD,)),
			('DeleteFileW',(LPVOID,)),
		],
	}

	
	def pre_DeleteFileW(self, event, ra, lpFileName):
		process, thread, context = get_state(event)
		print_hook('%s - DeleteFileW("%s")' % (hex(ra), process.peek_string(lpFileName)))
		return
	def post_DeleteFileW(self, event, retval):
		return

#######

dbg = winappdbg.Debug(JackPOSEventHandler(),bKillOnExit=True)

try:
	dbg.execl(debug_target)
	pid = dbg.get_debugee_pids()[0]
	print_script('Launched target: (%i) %s' % (pid, debug_target))
	print_script('Setting breakpoints')
	for bp in breakpoints:
		print_script('  %s -> %s' % (hex(bp[0]),bp[1].__name__))
		dbg.break_at(pid, bp[0], bp[1])
	dbg.loop()
finally:
	dbg.stop()

print_script('Target exited')