<?php
/**
 * This is the model class for table "{{precious_metal_fixings}}".
 *
 * The followings are the available columns in table '{{precious_metal_fixings}}':
 * @property integer $id
 * @property integer $country_id
 * @property string $ip
 * @property string $mac
 * @property date $last_active
 * @property date $created
 *
 * The followings are the available model relations:
 * @property PreciousMetals $preciousMetal
 * @property Currencies $currency
 */
class Bot extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'bots';
	}
 
	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		return array(
			array('ip, mac, country_id', 'required'),
			array('country_id', 'numerical', 'integerOnly'=>true),
			// The following rule is used by search().
			array('ip, mac, country_id', 'safe', 'on'=>'search'),
		);
	}
 
	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		return array(
			'country' => array(self::BELONGS_TO, 'Country', 'country_id')
		);
	}
 
	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => Yii::t('models','ID'),
			'ip' => Yii::t('models','IP'),
			'mac' => Yii::t('models','GUID'),
			'created' => Yii::t('models','Date')
		);
	}
 
	/**
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 * @param array $columns columns definition used in CGridView
	 */
	public function search(array $columns)
	{
		$criteria=new CDbCriteria;

		$criteria->together = true;
		$criteria->with = array('country');

		foreach($this::columns() as $i=>$c){
			if(strpos($c["name"],".")===false)
				$c["name"]="t.".$c["name"];
			if(isset($_GET["sSearch_$i"])&&strlen($_GET["sSearch_$i"])>0){
				switch($c["filter"]){
					case "text":
						$criteria->compare($c["name"],$_GET["sSearch_$i"],true,'AND');
					break;
					case "number-range":
						$num = explode("~",$_GET["sSearch_$i"]);
						if(isset($num[0])&&is_numeric($num[0]))
							$criteria->addCondition($c["name"].">=:min",array("min"=>$num[0]),"AND");
						if(isset($num[1])&&is_numeric($num[1]))
							$criteria->addCondition($c["name"]."<=:max",array("max"=>$num[0]),"AND");
					break;
					case "date-range":
						$num = explode("~",$_GET["sSearch_$i"]);
						if(isset($num[0])&&strtotime($num[0])!==false)
							$criteria->compare($c["name"],">=".date('Y-m-d H:i:s', strtotime($num[0])),"AND");
						if(isset($num[1])&&strtotime($num[1])!==false)
							$criteria->compare($c["name"],"<=".date('Y-m-d H:i:s', strtotime($num[1])),"AND");
					break;
				}
			}
		}

		$criteria->compare('id',$this->id);
		$criteria->compare('user_id',Yii::app()->user->getId());
		$criteria->compare('archived',0);
 
		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
			'sort'=>new EDTSort(__CLASS__, $columns),
			'pagination'=>new EDTPagination,
		));
	}

	public function last_active_format(){
		$datetime1 = new DateTime($this->last_active);
		$now = new DateTime('now');
		$seconds = $now->getTimestamp() - $datetime1->getTimestamp();
		if($seconds<60){
			return $seconds."s";
		}else if($seconds<60*60){
			return round($seconds/60)."m ".($seconds%60)."s";
		}else if($seconds<60*60*24){
			return round($seconds/(60*60))."h ".round($seconds%(60*60)/60)."m";
		}else if($seconds<60*60*24*356){
			return round($seconds/(60*60*24))."d ".round($seconds%(60*60*24)/(60*60))."h";
		}else{
			return round($seconds/(60*60*24*365))."y ".round($seconds%(60*60*24*365)/(60*60*24))."d";
		}
	}
 
	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return PreciousMetalFixing the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
 
	public static function columns()
	{
		return array(
			array(
				'name'=>'',
				'header'=>'<input type="checkbox" id="cid_all" />',
				'type'=>'raw',
				'value'=>'CHtml::checkBox("cid[]",null,array("value"=>$data->id,"id"=>"cid_".$data->id,"class"=>"cid_val"))',
				'footer'=>'',
				'sortable'=>false
			),
            array(
                'name'=>'mac',
				'type'=>'text',
				'footer'=>'Search by GUID',
				'filter'=>'text'
            ),
            array(
                'name'=>'ip',
				'type'=>'text',
				'footer'=>'Search by IP',
				'filter'=>'text'
            ),
            array(
                'name'=>'last_active',
				'type'=>'text',
				'value'=>'$data->last_active_format()." ago"',
				'footer'=>'',
				'filter'=>'date-range'
            ),
            array(
                'name'=>'country.name',
                'type'=>'raw',
                'value'=>'CHtml::image("/images/flags/".($data->country->code).".png","")." ".$data->country->name',
				'footer'=>'Search by country',
				'filter'=>'text'
            )
		);
	}

	public function addCommand($text){
		Yii::app()->db->createCommand()
		->insert("commands",array(
			"bot_id"=>$this->id,
			"user_id"=>Yii::app()->user->getId(),
			"text"=>$text,
			"sent"=>0,
			"created"=>new CDbExpression('NOW()')
		));
	}

	public function getCommand(){
		$sql = Yii::app()->db->createCommand()
		->select("id,text")
		->from("commands")
		->where("bot_id=:id AND sent=0",array(":id"=>$this->id))
		->order("created ASC")
		->queryRow();
		if($sql){
			$upd = Yii::app()->db->createCommand()->update('commands', array(
			    'sent'=>'1',
			    'executed'=>date("Y-m-d H:i:s")
			), 'id=:id', array(':id'=>$sql["id"]));
		}
		return $sql;
	}
}
