<?php
/**
 * This is the model class for table "{{precious_metal_fixings}}".
 *
 * The followings are the available columns in table '{{precious_metal_fixings}}':
 * @property integer $id
 * @property string $number
 * @property date $created
 *
 * The followings are the available model relations:
 * @property PreciousMetals $preciousMetal
 * @property Currencies $currency
 */
class Card extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'cards';
	}
 
	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		return array(
			array('number, created', 'required'),
			// The following rule is used by search().
			array('number, created', 'safe', 'on'=>'search'),
		);
	}
 
	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		return array(
			'tracks' => array(self::HAS_MANY , 'Tracks', 'card_id')
		);
	}
 
	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => Yii::t('models','ID'),
			'number' => Yii::t('models','Card number'),
			'created' => Yii::t('models','Date')
		);
	}
 
	/**
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 * @param array $columns columns definition used in CGridView
	 */
	public function search(array $columns)
	{
		$criteria=new CDbCriteria;
 
		if (isset($_GET['sSearch'])) {
			$criteria->compare('created',$_GET['sSearch'],true,'OR');
			$criteria->compare('number',$_GET['sSearch'],true,'OR');
		}
 
		$criteria->compare('id',$this->id);
 
		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
			'sort'=>new EDTSort(__CLASS__, $columns),
			'pagination'=>new EDTPagination,
		));
	}
 
	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return PreciousMetalFixing the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
 
	public static function columns()
	{
		return array(
			'id:number'
		);
	}
}